<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * UsersWP user title widget.
 *
 * @since 1.1.2
 */
class UWP_User_Title_Widget extends WP_Super_Duper {

    /**
     * Register the user title widget with WordPress.
     *
     */
    public function __construct() {


        $options = array(
            'textdomain'    => 'userswp',
            'block-icon'    => 'admin-site',
            'block-category'=> 'widgets',
            'block-keywords'=> "['userswp','user']",
            'class_name'     => __CLASS__,
            'base_id'       => 'uwp_user_title',
            'name'          => __('UWP > User Title','userswp'),
            'no_wrap'       => true,
            'widget_ops'    => array(
                'classname'   => 'uwp-user-title',
                'description' => esc_html__('Displays user name.','userswp'),
            ),
            'arguments'     => array(
                'tag'  => array(
                    'title' => __('Header Tag:', 'userswp'),
                    'desc' => __('Header tag for the user title.', 'userswp'),
                    'type' => 'select',
                    'options'   =>  array(
                        "h1" => "h1",
                        "h2" => "h2",
                        "h3" => "h3",
                        "h4" => "h4",
                        "h5" => "h5",
                        "h6" => "h6",
                    ),
                    'default'  => 'h2',
                    'desc_tip' => true,
                ),
                'user_id'  => array(
                    'title' => __('User ID:', 'userswp'),
                    'desc' => __('Leave blank to use current user ID. For profile page it will take displayed user ID', 'userswp'),
                    'type' => 'number',
                    'desc_tip' => true,
                    'default'  => '',
                    'advanced' => true
                ),
            )

        );


        parent::__construct( $options );
    }

    public function output( $args = array(), $widget_args = array(), $content = '' ) {

        $user = uwp_get_displayed_user();

        $defaults = array(
            'tag'      => 'h2',
        );

        $args = wp_parse_args( $args, $defaults );

        $args = apply_filters( 'uwp_widget_user_title_args', $args, $widget_args, $this );

        $title_tag = empty( $args['tag'] ) ? 'h2' : apply_filters( 'uwp_widget_user_title_tag', $args['tag'], $args, $widget_args, $this );
        if(empty($args['user_id']) && !empty($user->ID)){
            $args['user_id'] = $user->ID;
        }

        ob_start();

        do_action('uwp_user_title', $user, $title_tag);

        $output = ob_get_clean();

        return $output;

    }

}